summaryrefslogtreecommitdiff
path: root/src/index.erl
diff options
context:
space:
mode:
Diffstat (limited to 'src/index.erl')
-rw-r--r--src/index.erl64
1 files changed, 38 insertions, 26 deletions
diff --git a/src/index.erl b/src/index.erl
index 7871215..5169fbb 100644
--- a/src/index.erl
+++ b/src/index.erl
@@ -1,17 +1,18 @@
%%% Copyright (c) 2014, NORDUnet A/S.
%%% See LICENSE for licensing information.
%% Implements an interface to a file pair (basename and basename.chksum)
%% that stores an ordered list of fixed-size entries. Entries can be
%% added at the end and are retrieved by index. The list can also be
%% truncated.
%%
-%% Writes(add, truncate, addlast) need to be serialized.
+%% Writes(add, addlast) need to be serialized.
%% TODO: Checksums
-module(index).
--export([get/2, add/3, addlast/2, truncate/2]).
+-export([get/2, add/3, addlast/2]).
-define(ENTRYSIZE, 32).
-define(ENTRYSIZEINFILE, (?ENTRYSIZE*2+1)).
@@ -21,27 +22,38 @@ add(Basepath, Index, Entry) when is_binary(Entry), size(Entry) == ?ENTRYSIZE ->
case file:open(Basepath, [read, write, binary]) of
{ok, File} ->
{ok, Position} = file:position(File, eof),
- case Index of
- last when Position rem ?ENTRYSIZEINFILE == 0 ->
- ok;
- Index when is_integer(Index),
- Index * ?ENTRYSIZEINFILE == Position ->
- ok
- end,
+ Mode = case Index of
+ last when Position rem ?ENTRYSIZEINFILE == 0 ->
+ write;
+ Index when is_integer(Index),
+ Index * ?ENTRYSIZEINFILE == Position ->
+ write;
+ Index when is_integer(Index),
+ Index * ?ENTRYSIZEINFILE < Position ->
+ read;
+ _ ->
+ util:exit_with_error(invalid, writefile,
+ "Index not valid")
+ end,
EntryText = hex:bin_to_hexstr(Entry) ++ "\n",
- ok = file:write(File, EntryText),
- ok = file:close(File),
- util:fsync([Basepath, filename:dirname(Basepath)]);
- {error, Error} ->
- util:exit_with_error(Error, writefile,
- "Error opening file for writing")
- end.
-
-truncate(Basepath, Index) ->
- case file:open(Basepath, [read, write, binary]) of
- {ok, File} ->
- {ok, _Position} = file:position(File, Index * ?ENTRYSIZEINFILE),
- ok = file:truncate(File),
+ case Mode of
+ write ->
+ ok = file:write(File, EntryText);
+ read ->
+ {ok, _Position} =
+ file:position(File, {bof, Index * ?ENTRYSIZEINFILE}),
+ {ok, OldEntryText} = file:read(File, ?ENTRYSIZEINFILE),
+ %% check that the written content is the same as
+ %% the old content
+ case binary_to_list(OldEntryText) of
+ EntryText ->
+ ok;
+ _ ->
+ util:exit_with_error(invalid, writefile,
+ "Written content not the" ++
+ " same as old content")
+ end
+ end,
ok = file:close(File),
util:fsync([Basepath, filename:dirname(Basepath)]);
{error, Error} ->
@@ -1,17 +1,18 @@
%%% Copyright (c) 2014, NORDUnet A/S.
%%% See LICENSE for licensing information.
-%% Implements an interface to a file pair (basename and basename.chksum)
-%% that stores an ordered list of fixed-size entries. Entries can be
-%% added at the end and are retrieved by index. The list can also be
-%% truncated.
+%% Implements an interface to a file pair (basename and
+%% basename.chksum) that stores an ordered list of fixed-size entries.
+%% Entries can be added at the end and are retrieved by index. Entries
+%% can also be added at already existing indices, but then the
+%% contents must be the same.
%%